@@ -1,63 +1,101 @@
-//**********************************************************************
-//**********************************************************************
-//
-//  RANDOM SURVIVAL FOREST 3.5.1
-//
-//  Copyright 2008, Cleveland Clinic Foundation
-//
-//  This program is free software; you can redistribute it and/or
-//  modify it under the terms of the GNU General Public License
-//  as published by the Free Software Foundation; either version 2
-//  of the License, or (at your option) any later version.
-//
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//  GNU General Public License for more details.
-//
-//  You should have received a copy of the GNU General Public
-//  License along with this program; if not, write to the Free
-//  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-//  Boston, MA  02110-1301, USA.
-//
-//  Project funded by:
-//    National Institutes of Health, HL072771-01
-//
-//    Michael S Lauer, MD, FACC, FAHA
-//    Cleveland Clinic Lerner College of Medicine of CWRU
-//    9500 Euclid Avenue
-//    Cleveland, OH 44195
-//
-//    email:  lauerm@ccf.org
-//    phone:   216-444-6798
-//
-//  Written by:
-//    Hemant Ishwaran, Ph.D.
-//    Dept of Quantitative Health Sciences/Wb4
-//    Cleveland Clinic Foundation
-//    9500 Euclid Avenue
-//    Cleveland, OH 44195
-//
-//    email:  hemant.ishwaran@gmail.com
-//    phone:  216-444-9932
-//    URL:    www.bio.ri.ccf.org/Resume/Pages/Ishwaran/ishwaran.html
-//    --------------------------------------------------------------
-//    Udaya B. Kogalur, Ph.D.
-//    Kogalur Shear Corporation
-//    5425 Nestleway Drive, Suite L1
-//    Clemmons, NC 27012
-//
-//    email:  ubk2101@columbia.edu
-//    phone:  919-824-9825
-//    URL:    www.kogalur-shear.com
-//
-//**********************************************************************
-//**********************************************************************
+////**********************************************************************
+////**********************************************************************
+////
+////  RANDOM SURVIVAL FOREST 3.6.0
+////
+////  Copyright 2009, Cleveland Clinic Foundation
+////
+////  This program is free software; you can redistribute it and/or
+////  modify it under the terms of the GNU General Public License
+////  as published by the Free Software Foundation; either version 2
+////  of the License, or (at your option) any later version.
+////
+////  This program is distributed in the hope that it will be useful,
+////  but WITHOUT ANY WARRANTY; without even the implied warranty of
+////  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+////  GNU General Public License for more details.
+////
+////  You should have received a copy of the GNU General Public
+////  License along with this program; if not, write to the Free
+////  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+////  Boston, MA  02110-1301, USA.
+////
+////  ----------------------------------------------------------------
+////  Project Partially Funded By:
+////    --------------------------------------------------------------
+////    National Institutes of Health,  Grant HHSN268200800026C/0001
+////
+////    Michael S. Lauer, M.D., FACC, FAHA 
+////    National Heart, Lung, and Blood Institute
+////    6701 Rockledge Dr, Room 10122
+////    Bethesda, MD 20892
+////
+////    email:  lauerm@nhlbi.nih.gov
+////
+////    --------------------------------------------------------------
+////    Case Western Reserve University/Cleveland Clinic  
+////    CTSA Grant:  UL1 RR024989, National Center for
+////    Research Resources (NCRR), NIH
+////
+////    --------------------------------------------------------------
+////    Dept of Defense Era of Hope Scholar Award, Grant W81XWH0910339
+////    Andy Minn, M.D., Ph.D.
+////    Department of Radiation and Cellular Oncology, and
+////    Ludwig Center for Metastasis Research
+////    The University of Chicago, Jules F. Knapp Center, 
+////    924 East 57th Street, Room R318
+////    Chicago, IL 60637
+//// 
+////    email:  aminn@radonc.uchicago.edu
+////
+////    --------------------------------------------------------------
+////    Bryan Lau, Ph.D.
+////    Department of Medicine, Johns Hopkins School of Medicine,
+////    Baltimore, Maryland 21287
+////
+////    email:  blau1@jhmi.edu
+////
+////  ----------------------------------------------------------------
+////  Written by:
+////    --------------------------------------------------------------
+////    Hemant Ishwaran, Ph.D.
+////    Dept of Quantitative Health Sciences/Wb4
+////    Cleveland Clinic Foundation
+////    9500 Euclid Avenue
+////    Cleveland, OH 44195
+////
+////    email:  hemant.ishwaran@gmail.com
+////    phone:  216-444-9932
+////    URL:    www.bio.ri.ccf.org/Resume/Pages/Ishwaran/ishwaran.html
+////
+////    --------------------------------------------------------------
+////    Udaya B. Kogalur, Ph.D.
+////    Dept of Quantitative Health Sciences/Wb4
+////    Cleveland Clinic Foundation
+////    
+////    Kogalur Shear Corporation
+////    5425 Nestleway Drive, Suite L1
+////    Clemmons, NC 27012
+////
+////    email:  ubk2101@columbia.edu
+////    phone:  919-824-9825
+////    URL:    www.kogalur-shear.com
+////    --------------------------------------------------------------
+////
+////**********************************************************************
+////**********************************************************************
 
-#include   "global.h"
-#include   "nrutil.h"
-extern int  *_seed2Ptr;
-extern uint getTraceFlag();
+#include      <stdlib.h>
+#include      "nrutil.h"
+#include <R_ext/Print.h>
+#define NUMR_DEF_TRACE  0x002000  
+extern unsigned int getTraceFlag();
+#ifndef TRUE
+#define TRUE   0x01
+#endif
+#ifndef FALSE
+#define FALSE  0x00
+#endif
 #define IA 16807
 #define IM 2147483647
 #define AM (1.0/IM)
@@ -144,9 +182,9 @@ float ran2(int *idum) {
 #undef NDIV
 #undef EPS
 #undef RNMX
-uint upower (uint x, uint n) {
-  uint p;
-  if ((x >= 2) & (n > (sizeof(uint) * 8))) {
+unsigned int upower (unsigned int x, unsigned int n) {
+  unsigned int p;
+  if ((x >= 2) & (n > (sizeof(unsigned int) * 8))) {
     nrerror("Overflow in upower(), exponent too large.");
   }
   for (p = 1; n > 0; --n) {
@@ -154,16 +192,16 @@ uint upower (uint x, uint n) {
   }
   return p;
 }
-uint upower2 (uint n) {
-  uint p;
-  if (n > (sizeof(uint) * 8)) {
+unsigned int upower2 (unsigned int n) {
+  unsigned int p;
+  if (n > (sizeof(unsigned int) * 8)) {
     nrerror("Overflow in upower2(), exponent too large.");
   }
   p = 1 << n;
   return p;
 }
-uint ulog2 (uint n) {
-  uint p;
+unsigned int ulog2 (unsigned int n) {
+  unsigned int p;
   p = 0;
   while (n > 1) {
     n = n >> 1;
@@ -171,8 +209,8 @@ uint ulog2 (uint n) {
   }
   return p;
 }
-void hpsort(double *ra, uint n) {
-  uint i, ir, j, l;
+void hpsort(double *ra, unsigned int n) {
+  unsigned int i, ir, j, l;
   double rra;
   if (n < 2) return;
   l=(n >> 1)+1;
@@ -205,9 +243,9 @@ void hpsort(double *ra, uint n) {
     ra[i] = rra;
   }
 }
-void hpsortui(uint *ra, uint n) {
-  uint i, ir, j, l;
-  uint rra;
+void hpsortui(unsigned int *ra, unsigned int n) {
+  unsigned int i, ir, j, l;
+  unsigned int rra;
   if (n < 2) return;
   l=(n >> 1)+1;
   ir=n;
@@ -245,10 +283,10 @@ void hpsortui(uint *ra, uint n) {
 #define SWAP(a,b) itemp=(a);(a)=(b);(b)=itemp;
 #define M 7
 #define NSTACK 50
-void indexx(uint n, double *arr, uint *indx) {
-  uint i, j, k, l=1;
-  uint indxt, itemp, ir=n;
-  uint *istack, jstack=0;
+void indexx(unsigned int n, double *arr, unsigned int *indx) {
+  unsigned int i, j, k, l=1;
+  unsigned int indxt, itemp, ir=n;
+  unsigned int *istack, jstack=0;
   double a;
   istack = uivector(1, NSTACK);
   for (j=1; j<=n; j++) indx[j]=j;
@@ -310,37 +348,77 @@ void indexx(uint n, double *arr, uint *indx) {
 #undef SWAP
 #undef M
 #undef NSTACK
-void permute(uint n, uint *indx) {
-  uint i,j,k;
-  for (i=1; i<= n; i++) {
-    indx[i] = 0;
-  }
-  for (i=n; i > 0; i--) {
-    k = (uint) ceil(ran2(_seed2Ptr)*(i*1.0));
-    for (j = 1; k > 0; j++) {
-      if (indx[j] == 0) {
-        k--;
-      }
-    }
-    indx[j-1] = i;
-  }
-  if (getTraceFlag() & NUMR_DEF_TRACE) {
-    Rprintf("\nRequest for Permutation:  \n");
-    Rprintf("     index   permute\n");
-    for (i=1; i <= n; i++) {
-      Rprintf("%10d %10d \n", i, indx[i]);
-    }
-  }
-}
 #define FREE_ARG char*
 #define NR_END 1
-double ***dmatrix3(ulong n3l, ulong n3h, ulong nrl, ulong nrh, ulong ncl, ulong nch) {
-  ulong i,j,k,  n3 = n3h - n3l + 1, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
+double ****dmatrix4(unsigned long n4l,
+                    unsigned long n4h,
+                    unsigned long n3l, 
+                    unsigned long n3h, 
+                    unsigned long nrl, 
+                    unsigned long nrh, 
+                    unsigned long ncl, 
+                    unsigned long nch) {
+  unsigned long i, j, k, p;
+  unsigned long n4   = n4h - n4l + 1;
+  unsigned long n3   = n3h - n3l + 1; 
+  unsigned long nrow = nrh - nrl + 1; 
+  unsigned long ncol = nch - ncl + 1;
+  double ****m;
+  m=(double ****) malloc((size_t)((n4+NR_END)*sizeof(double***)));
+  if (!m) nrerror("Allocation Failure 1 in dmatrix4()");
+  m += NR_END;
+  m -= n4l;
+  m[n4l]=(double ***) malloc((size_t)((n4*n3+NR_END)*sizeof(double**)));
+  if (!m[n4l]) nrerror("Allocation Failure 2 in dmatrix4()");
+  m[n4l] += NR_END;
+  m[n4l] -= n3l;
+  m[n4l][n3l]=(double **) malloc((size_t)((n4*n3*nrow+NR_END)*sizeof(double*)));
+  if (!m[n4l][n3l]) nrerror("Allocation Failure 3 in dmatrix4()");
+  m[n4l][n3l] += NR_END;
+  m[n4l][n3l] -= nrl;
+  m[n4l][n3l][nrl]=(double *) malloc((size_t)((n4*n3*nrow*ncol+NR_END)*sizeof(double)));
+  if (!m[n4l][n3l][nrl]) nrerror("Allocation Failure 4 in dmatrix4()");
+  m[n4l][n3l][nrl] += NR_END;
+  m[n4l][n3l][nrl] -= ncl;
+  for(i=n4l+1;i<=n4h;i++) {
+    m[i] = m[i-1] + n3;
+  }
+  p = 0;
+  for(i=n4l;i<=n4h;i++) {
+    for(j=n3l;j<=n3h;j++) {
+      m[i][j] = m[n4l][n3l] + nrow * p;
+      p++;
+    }
+  }
+  p = 0;
+  for(i=n4l;i<=n4h;i++) {
+    for(j=n3l;j<=n3h;j++) {
+      for(k=nrl;k<=nrh;k++) {
+        m[i][j][k] = m[n4l][n3l][nrl] + ncol * p;
+        p++;
+      }
+    }
+  }
+  if (getTraceFlag() & NUMR_DEF_TRACE) {
+    Rprintf("\ndmatrix4() alloc:  %20x  \n", m);
+  }
+  return m;
+}
+double ***dmatrix3(unsigned long n3l, 
+                   unsigned long n3h, 
+                   unsigned long nrl, 
+                   unsigned long nrh, 
+                   unsigned long ncl, 
+                   unsigned long nch) {
+  unsigned long i, j, k;  
+  unsigned long n3   = n3h - n3l + 1;
+  unsigned long nrow = nrh - nrl + 1; 
+  unsigned long ncol = nch - ncl + 1;
   double ***m;
   m=(double ***) malloc((size_t)((n3+NR_END)*sizeof(double**)));
   if (!m) nrerror("Allocation Failure 1 in dmatrix3()");
   m += NR_END;
-  m -= nrl;
+  m -= n3l;
   m[n3l]=(double **) malloc((size_t)((n3*nrow+NR_END)*sizeof(double*)));
   if (!m[n3l]) nrerror("Allocation Failure 2 in dmatrix3()");
   m[n3l] += NR_END;
@@ -364,8 +442,8 @@ double ***dmatrix3(ulong n3l, ulong n3h, ulong nrl, ulong nrh, ulong ncl, ulong
   }
   return m;
 }
-double **dmatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
-  ulong i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
+double **dmatrix(unsigned long nrl, unsigned long nrh, unsigned long ncl, unsigned long nch) {
+  unsigned long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
   double **m;
   m=(double **) malloc((size_t)((nrow+NR_END)*sizeof(double*)));
   if (!m) nrerror("Allocation Failure 1 in dmatrix()");
@@ -381,8 +459,8 @@ double **dmatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
   }
   return m;
 }
-int **imatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
-  ulong i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
+int **imatrix(unsigned long nrl, unsigned long nrh, unsigned long ncl, unsigned long nch) {
+  unsigned long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
   int **m;
   m=(int **) malloc((size_t)((nrow+NR_END)*sizeof(int*)));
   if (!m) nrerror("Allocation Failure 1 in imatrix()");
@@ -398,14 +476,14 @@ int **imatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
   }
   return m;
 }
-uint **uimatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
-  ulong i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
-  uint **m;
-  m=(uint **) malloc((size_t)((nrow+NR_END)*sizeof(uint*)));
+unsigned int **uimatrix(unsigned long nrl, unsigned long nrh, unsigned long ncl, unsigned long nch) {
+  unsigned long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
+  unsigned int **m;
+  m=(unsigned int **) malloc((size_t)((nrow+NR_END)*sizeof(unsigned int*)));
   if (!m) nrerror("Allocation Failure 1 in uimatrix()");
   m += NR_END;
   m -= nrl;
-  m[nrl]=(uint *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(uint)));
+  m[nrl]=(unsigned int *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(unsigned int)));
   if (!m[nrl]) nrerror("Allocation Failure 2 in uimatrix()");
   m[nrl] += NR_END;
   m[nrl] -= ncl;
@@ -415,14 +493,14 @@ uint **uimatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
   }
   return m;
 }
-ulong **ulmatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
-  ulong i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
-  ulong **m;
-  m=(ulong **) malloc((size_t)((nrow+NR_END)*sizeof(ulong*)));
+unsigned long **ulmatrix(unsigned long nrl, unsigned long nrh, unsigned long ncl, unsigned long nch) {
+  unsigned long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
+  unsigned long **m;
+  m=(unsigned long **) malloc((size_t)((nrow+NR_END)*sizeof(unsigned long*)));
   if (!m) nrerror("Allocation Failure 1 in ulmatrix()");
   m += NR_END;
   m -= nrl;
-  m[nrl]=(ulong *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(ulong)));
+  m[nrl]=(unsigned long *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(unsigned long)));
   if (!m[nrl]) nrerror("Allocation Failure 2 in ulmatrix()");
   m[nrl] += NR_END;
   m[nrl] -= ncl;
@@ -432,8 +510,8 @@ ulong **ulmatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
   }
   return m;
 }
-char **cmatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
-  ulong i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
+char **cmatrix(unsigned long nrl, unsigned long nrh, unsigned long ncl, unsigned long nch) {
+  unsigned long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
   char **m;
   m=(char **) malloc((size_t)((nrow+NR_END)*sizeof(char*)));
   if (!m) nrerror("Allocation Failure 1 in cmatrix()");
@@ -449,7 +527,7 @@ char **cmatrix(ulong nrl, ulong nrh, ulong ncl, ulong nch) {
   }
   return m;
 }
-double *dvector(ulong nl, ulong nh) {
+double *dvector(unsigned long nl, unsigned long nh) {
   double *v;
   v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
   if (!v) nrerror("Allocation Failure in dvector()");
@@ -459,7 +537,7 @@ double *dvector(ulong nl, ulong nh) {
   }
   return v;
 }
-double **pdvector(ulong nl, ulong nh) {
+double **pdvector(unsigned long nl, unsigned long nh) {
   double **v;
   v=(double **) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double*)));
   if (!v) nrerror("Allocation Failure in pdvector()");
@@ -469,17 +547,27 @@ double **pdvector(ulong nl, ulong nh) {
   }
   return v;
 }
-int *ivector(ulong nl, ulong nh) {
+double ***ppdvector(unsigned long nl, unsigned long nh) {
+  double ***v;
+  v=(double ***) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double**)));
+  if (!v) nrerror("Allocation Failure in ppdvector()");
+  v = v-nl+NR_END;
+  if (getTraceFlag() & NUMR_DEF_TRACE) {
+    Rprintf("\nppdvector() alloc:  %20x  \n", v);
+  }
+  return v;
+}
+int *ivector(unsigned long nl, unsigned long nh) {
   int *v;
   v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
-  if (!v) nrerror("Allocation Failure in uivector()");
+  if (!v) nrerror("Allocation Failure in ivector()");
   v = v-nl+NR_END;
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\nivector() alloc:  %20x  \n", v);
   }
   return v;
 }
-int **pivector(ulong nl, ulong nh) {
+int **pivector(unsigned long nl, unsigned long nh) {
   int **v;
   v=(int **) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int*)));
   if (!v) nrerror("Allocation Failure in puivector()");
@@ -489,9 +577,9 @@ int **pivector(ulong nl, ulong nh) {
   }
   return v;
 }
-uint *uivector(ulong nl, ulong nh) {
-  uint *v;
-  v=(uint *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(uint)));
+unsigned int *uivector(unsigned long nl, unsigned long nh) {
+  unsigned int *v;
+  v=(unsigned int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned int)));
   if (!v) nrerror("Allocation Failure in uivector()");
   v = v-nl+NR_END;
   if (getTraceFlag() & NUMR_DEF_TRACE) {
@@ -499,9 +587,9 @@ uint *uivector(ulong nl, ulong nh) {
   }
   return v;
 }
-uint **puivector(ulong nl, ulong nh) {
-  uint **v;
-  v=(uint **) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(uint*)));
+unsigned int **puivector(unsigned long nl, unsigned long nh) {
+  unsigned int **v;
+  v=(unsigned int **) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned int*)));
   if (!v) nrerror("Allocation Failure in puivector()");
   v = v-nl+NR_END;
   if (getTraceFlag() & NUMR_DEF_TRACE) {
@@ -509,9 +597,9 @@ uint **puivector(ulong nl, ulong nh) {
   }
   return v;
 }
-uint ***ppuivector(ulong nl, ulong nh) {
-  uint ***v;
-  v=(uint ***) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(uint**)));
+unsigned int ***ppuivector(unsigned long nl, unsigned long nh) {
+  unsigned int ***v;
+  v=(unsigned int ***) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned int**)));
   if (!v) nrerror("Allocation Failure in ppuivector()");
   v = v-nl+NR_END;
   if (getTraceFlag() & NUMR_DEF_TRACE) {
@@ -519,9 +607,9 @@ uint ***ppuivector(ulong nl, ulong nh) {
   }
   return v;
 }
-ulong *ulvector(ulong nl, ulong nh) {
-  ulong *v;
-  v=(ulong *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(ulong)));
+unsigned long *ulvector(unsigned long nl, unsigned long nh) {
+  unsigned long *v;
+  v=(unsigned long *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned long)));
   if (!v) nrerror("Allocation Failure in ulvector()");
   v = v-nl+NR_END;
   if (getTraceFlag() & NUMR_DEF_TRACE) {
@@ -529,9 +617,9 @@ ulong *ulvector(ulong nl, ulong nh) {
   }
   return v;
 }
-ulong **pulvector(ulong nl, ulong nh) {
-  ulong **v;
-  v=(ulong **) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(ulong*)));
+unsigned long **pulvector(unsigned long nl, unsigned long nh) {
+  unsigned long **v;
+  v=(unsigned long **) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned long*)));
   if (!v) nrerror("Allocation Failure in pulvector()");
   v = v-nl+NR_END;
   if (getTraceFlag() & NUMR_DEF_TRACE) {
@@ -539,7 +627,7 @@ ulong **pulvector(ulong nl, ulong nh) {
   }
   return v;
 }
-char *cvector(ulong nl, ulong nh) {
+char *cvector(unsigned long nl, unsigned long nh) {
   char *v;
   v=(char *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(char)));
   if (!v) nrerror("Allocation Failure in cvector()");
@@ -549,7 +637,7 @@ char *cvector(ulong nl, ulong nh) {
   }
   return v;
 }
-char **pcvector(ulong nl, ulong nh) {
+char **pcvector(unsigned long nl, unsigned long nh) {
   char **v;
   v=(char **) malloc((size_t) ((nh-nl+1+NR_END)*sizeof(char*)));
   if (!v) nrerror("Allocation Failure in pcvector()");
@@ -565,15 +653,33 @@ void nrerror(char error_text[]) {
   Rprintf("\nRSF:  %s", error_text);
   Rprintf("\nRSF:  Please Contact Technical Support.");
   Rprintf("\nRSF:  The application will now exit.\n");
-  exit(FALSE);
+  exit(TRUE);
+}
+void free_dmatrix4(double    ****m,
+                   unsigned long n4l,
+                   unsigned long n4h,
+                   unsigned long n3l,
+                   unsigned long n3h,
+                   unsigned long nrl,
+                   unsigned long nrh,
+                   unsigned long ncl,
+                   unsigned long nch
+                   ) {
+  if (getTraceFlag() & NUMR_DEF_TRACE) {
+    Rprintf("\ndmatrix4() de-allocating:  %20x  \n", m);
+  }
+  free((FREE_ARG) (m[n4l][n3l][nrl]+ncl-NR_END));
+  free((FREE_ARG) (m[n4l][n3l]+nrl-NR_END));
+  free((FREE_ARG) (m[n4l]+n3l-NR_END));
+  free((FREE_ARG) (m+n4l-NR_END));
 }
 void free_dmatrix3(double ***m,
-                  ulong n3l,
-                  ulong n3h,
-                  ulong nrl,
-                  ulong nrh,
-                  ulong ncl,
-                  ulong nch
+                  unsigned long n3l,
+                  unsigned long n3h,
+                  unsigned long nrl,
+                  unsigned long nrh,
+                  unsigned long ncl,
+                  unsigned long nch
                  ) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\ndmatrix3() de-allocating:  %20x  \n", m);
@@ -583,10 +689,10 @@ void free_dmatrix3(double ***m,
   free((FREE_ARG) (m+n3l-NR_END));
 }
 void free_dmatrix(double **m,
-                  ulong nrl,
-                  ulong nrh,
-                  ulong ncl,
-                  ulong nch
+                  unsigned long nrl,
+                  unsigned long nrh,
+                  unsigned long ncl,
+                  unsigned long nch
                  ) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\ndmatrix() de-allocating:  %20x  \n", m);
@@ -595,10 +701,10 @@ void free_dmatrix(double **m,
   free((FREE_ARG) (m+nrl-NR_END));
 }
 void free_imatrix(int **m,
-                  ulong nrl,
-                  ulong nrh,
-                  ulong ncl,
-                  ulong nch
+                  unsigned long nrl,
+                  unsigned long nrh,
+                  unsigned long ncl,
+                  unsigned long nch
                   ) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\nimatrix() de-allocating:  %20x  \n", m);
@@ -606,11 +712,11 @@ void free_imatrix(int **m,
   free((FREE_ARG) (m[nrl]+ncl-NR_END));
   free((FREE_ARG) (m+nrl-NR_END));
 }
-void free_uimatrix(uint **m,
-                   ulong nrl,
-                   ulong nrh,
-                   ulong ncl,
-                   ulong nch
+void free_uimatrix(unsigned int **m,
+                   unsigned long nrl,
+                   unsigned long nrh,
+                   unsigned long ncl,
+                   unsigned long nch
                   ) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\nuimatrix() de-allocating:  %20x  \n", m);
@@ -618,11 +724,11 @@ void free_uimatrix(uint **m,
   free((FREE_ARG) (m[nrl]+ncl-NR_END));
   free((FREE_ARG) (m+nrl-NR_END));
 }
-void free_ulmatrix(ulong **m,
-                   ulong nrl,
-                   ulong nrh,
-                   ulong ncl,
-                   ulong nch
+void free_ulmatrix(unsigned long **m,
+                   unsigned long nrl,
+                   unsigned long nrh,
+                   unsigned long ncl,
+                   unsigned long nch
                   ) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\nulmatrix() de-allocating:  %20x  \n", m);
@@ -631,10 +737,10 @@ void free_ulmatrix(ulong **m,
   free((FREE_ARG) (m+nrl-NR_END));
 }
 void free_cmatrix(char **m,
-                  ulong nrl,
-                  ulong nrh,
-                  ulong ncl,
-                  ulong nch
+                  unsigned long nrl,
+                  unsigned long nrh,
+                  unsigned long ncl,
+                  unsigned long nch
                   ) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\ncmatrix() de-allocating:  %20x  \n", m);
@@ -642,67 +748,73 @@ void free_cmatrix(char **m,
   free((FREE_ARG) (m[nrl]+ncl-NR_END));
   free((FREE_ARG) (m+nrl-NR_END));
 }
-void free_dvector(double *v, ulong nl, ulong nh) {
+void free_dvector(double *v, unsigned long nl, unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\ndvector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_pdvector(double **v, ulong nl, ulong nh) {
+void free_pdvector(double **v, unsigned long nl, unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\npdvector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_ivector(int *v,ulong nl,ulong nh) {
+void free_ppdvector(double ***v, unsigned long nl, unsigned long nh) {
+  if (getTraceFlag() & NUMR_DEF_TRACE) {
+    Rprintf("\nppdvector() de-allocating:  %20x  \n", v);
+  }
+  free((FREE_ARG) (v+nl-NR_END));
+}
+void free_ivector(int *v,unsigned long nl,unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\nivector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_pivector(int **v, ulong nl, ulong nh) {
+void free_pivector(int **v, unsigned long nl, unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\npivector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_uivector(uint *v,ulong nl,ulong nh) {
+void free_uivector(unsigned int *v,unsigned long nl,unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\nuivector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_puivector(uint **v, ulong nl, ulong nh) {
+void free_puivector(unsigned int **v, unsigned long nl, unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\npuivector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_ppuivector(uint ***v, ulong nl, ulong nh) {
+void free_ppuivector(unsigned int ***v, unsigned long nl, unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\nppuivector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_ulvector(ulong *v,ulong nl,ulong nh) {
+void free_ulvector(unsigned long *v,unsigned long nl,unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\nulvector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_pulvector(ulong **v, ulong nl, ulong nh) {
+void free_pulvector(unsigned long **v, unsigned long nl, unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\npulvector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_cvector(char *v,ulong nl,ulong nh) {
+void free_cvector(char *v,unsigned long nl,unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\ncvector() de-allocating:  %20x  \n", v);
   }
   free((FREE_ARG) (v+nl-NR_END));
 }
-void free_pcvector(char **v, ulong nl, ulong nh) {
+void free_pcvector(char **v, unsigned long nl, unsigned long nh) {
   if (getTraceFlag() & NUMR_DEF_TRACE) {
     Rprintf("\npcvector() de-allocating:  %20x  \n", v);
   }
@@ -710,27 +822,26 @@ void free_pcvector(char **v, ulong nl, ulong nh) {
 }
 #undef FREE_ARG
 #undef NR_END
-void nrCopyMatrix(uint **new, uint **old, uint nrow, uint ncol) {
-  uint i,j;
+void nrCopyMatrix(unsigned int **new, unsigned int **old, unsigned int nrow, unsigned int ncol) {
+  unsigned int i,j;
   for (i = 1; i <= nrow; i++) {  
     for (j = 1; j <= ncol; j++) {  
       new[i][j] = old[i][j];
     }
   }
 }
-void nrCopyVector(char *new, char *old, uint ncol) {
-  uint j;
+void nrCopyVector(char *new, char *old, unsigned int ncol) {
+  unsigned int j;
   for (j = 1; j <= ncol; j++) {  
     new[j] = old[j];
   }
 }
 void testEndianness() {
-  uint test = 0x12345678;
-  uint *testPtr = & test;
+  unsigned int     test = 0x12345678;
+  unsigned int *testPtr = & test;
   Rprintf("\n %2x %2x %2x %2x \n", 
           *((char *) testPtr), 
           *((char *) testPtr + 1), 
           *((char *) testPtr + 2), 
           *((char *) testPtr + 3));
-  exit(0);
 }
